/* Host configuration file for Interactive Unix 3.2 */
/* From Minh Tran-Le <TRANLE@INTELLICORP.COM>.  */
#include <ansidecl.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>

#ifndef O_ACCMODE
#define O_ACCMODE (O_RDONLY | O_WRONLY | O_RDWR)
#endif
#ifndef SEEK_SET
#define SEEK_SET 0
#define SEEK_CUR 1
#endif

#define USE_UTIME

/* Some things that need to be defined in order to make code written for
   BSD Unix compile under System V Unix.  */

/*#include <memory.h>*/
#define bcmp(b1,b2,len)		memcmp(b1,b2,len)
#define bcopy(src,dst,len)	memcpy(dst,src,len)
#define bzero(s,n)		memset(s,0,n)

#include <string.h>

#ifndef DONTDECLARE_MALLOC
extern PTR  EXFUN(malloc,(unsigned));
extern PTR  EXFUN(realloc, (PTR, unsigned));
extern void EXFUN(free,(PTR));
#endif

#include "fopen-same.h"

#ifndef fileno
#define fileno(fp) ((fp)->_file)
#endif
